'use strict';
/**
 * A fellow with similarties to other people.
 * @property {string} homepage
 * @property {string} githubUrl
 * @property {string} githubUsername
 * @property {string} twitterUrl
 * @property {string} twitterUsername
 * @property {string} facebookUrl
 * @property {string} facebookUsername
 * @class
 */

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Fellow =
/*#__PURE__*/
function () {
  _createClass(Fellow, [{
    key: "compare",

    /**
    Compare a fellow to another fellow, uses ensureFields for the comparison
    If an ensureField is an array for both ourself and the other fellow, we will check to see if only one item of that array is similar (this is useful for email comparisons)
    @param {Fellow} other The other fellow to compare ourselves with
    @returns {Boolean} Returns `true` if they appear to be the same person, or `false` if not
    */
    value: function compare(other) {
      var fields = this.ensureFields;
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = fields[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var field = _step.value;
          var value = this[field];
          var otherValue = other[field];

          if (value && otherValue) {
            if (Array.isArray(value) && Array.isArray(otherValue)) {
              var _iteratorNormalCompletion2 = true;
              var _didIteratorError2 = false;
              var _iteratorError2 = undefined;

              try {
                for (var _iterator2 = value[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                  var item = _step2.value;

                  if (otherValue.indexOf(item) !== -1) {
                    return true;
                  }
                }
              } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
              } finally {
                try {
                  if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
                    _iterator2.return();
                  }
                } finally {
                  if (_didIteratorError2) {
                    throw _iteratorError2;
                  }
                }
              }
            } else if (value === otherValue) {
              return true;
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return false;
    }
    /**
    Get a fellow from the singleton list
    @method get
    @param {*} value The value to fetch the value with
    @returns {Fellow|Null} The fetched fellow, if they exist with that value
    */

  }, {
    key: "ensureFields",

    /**
    An array of fields that will denote if the user is the same user or not when using ensure
    @access protected
    @type {Array}
    */
    get: function get() {
      return ['emails', 'homepage', 'githubUsername', 'twitterUsername', 'facebookUsername'];
    }
  }, {
    key: "json",
    // -----------------------------------
    // Properties

    /**
    Get the serialisable values of this fellow
    @type {Object}
    */
    get: function get() {
      var json = JSON.parse(JSON.stringify(this));
      return json;
    }
    /**
    When fetching,
    Updating the email will add the email to the start of the `emails` array.
    	When getting,
    Fetches the first email from the emails array.
    @property {String} email
    @param {String} value - The email to add to the emails
    */

  }, {
    key: "email",
    set: function set(value) {
      var index = this.emails.indexOf(value);

      if (index !== -1) {
        this.emails = this.emails.slice(0, index).concat(this.emails.slice(index + 1));
      }

      this.emails.unshift(value);
    },
    get: function get() {
      return this.emails[0] || null;
    }
    /**
    When setting,
    Will determine passed url is a github, facebook, or twitter url.
    If it is, then it will extract the username and url from it.
    If this is not one of those urls, then it will set the homepage variable.
    	When getting,
    Will fetch `homepage || githubUrl || facebookUrl || twitterUrl || null`
    @type {String}
    @param {String} value
    @returns {this}
    */

  }, {
    key: "url",
    set: function set(value) {
      var githubMatch = /^.+github.com\/([^/]+)\/?$/.exec(value);

      if (githubMatch) {
        this.githubUsername = githubMatch[1];
        this.githubUrl = 'https://github.com/' + this.githubUsername;
      } else {
        var facebookMatch = /^.+facebook.com\/([^/]+)\/?$/.exec(value);

        if (facebookMatch) {
          this.facebookUsername = facebookMatch[1];
          this.facebookUrl = 'https://facebook.com/' + this.facebookUsername;
        } else {
          var twitterMatch = /^.+twitter.com\/([^/]+)\/?$/.exec(value);

          if (twitterMatch) {
            this.twitterUsername = twitterMatch[1];
            this.twitterUrl = 'https://twitter.com/' + this.twitterUsername;
          } else {
            this.homepage = value.replace(/^https/, 'http');
          }
        }
      }
    },
    get: function get() {
      return this.homepage || this.githubUrl || this.facebookUrl || this.twitterUrl || null;
    } // -----------------------------------
    // Methods

    /**
    Construct our fellow instance with the value
    @param {*} value - The value used to set the properties of the fellow, forwarded to set
    */

  }], [{
    key: "create",
    // -----------------------------------
    // Static Helpers

    /**
    Create a new Fellow instance with the value, however if the value is already a fellow instance, then just return it
    @static
    @param {*} value
    @returns {Fellow} The new fellow instance
    */
    value: function create(value) {
      return value instanceof this ? value : new this(value);
    }
    /**
    A singleton array attached to the class object that stores it's people objects
    @static
    @type {Array}
    */

  }, {
    key: "ensure",

    /**
    With the value, see if an existing fellow exists in our singleton list property with the value, otherwise create a new fellow instance with the value and add them to our singleton list
    @static
    @param {*} value - The value to create a new fellow instance or find the existing fellow instance with
    @param {boolean} [add=true] - Whether to add the created person to the list
    @returns {Fellow} The new or existing fellow instance
    */
    value: function ensure(value) {
      var add = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
      var newFellow = this.create(value);
      var people = this.list;
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = people[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var existingFellow = _step3.value;

          if (newFellow.compare(existingFellow)) {
            return existingFellow.set(value);
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      if (add) {
        people.push(newFellow);
        return newFellow;
      } else {
        return null;
      }
    }
  }, {
    key: "get",
    value: function get(value) {
      return this.ensure(value, false);
    }
    /**
    Add a fellow or a series of people, denoted by the value, to the singleton list
    @static
    @param {*} value The fellow or people to add
    @returns {Array} An array of the fellow objects for the passed people
    */

  }, {
    key: "add",
    value: function add(value) {
      var _this = this;

      if (value instanceof this) {
        return [this.ensure(value)];
      } else if (typeof value === 'string') {
        return value.split(/, +/).map(function (fellow) {
          return _this.ensure(fellow);
        });
      } else if (Array.isArray(value)) {
        return value.map(function (value) {
          return _this.ensure(value);
        });
      } else if (value) {
        return [this.ensure(value)];
      } else {
        return [];
      }
    }
    /**
    Get all fellows who contribute to a particular repository
    @static
    @param {String} repoSlug The repository slug
    @returns {Array} An array of the fellow objects that contribute to the repository
    */

  }, {
    key: "contributesRepository",
    value: function contributesRepository(repoSlug) {
      return this.list.filter(function (fellow) {
        return fellow.ensureRepository(repoSlug).contributes;
      });
    }
    /**
    Get all fellows who maintain a particular repository
    @static
    @param {String} repoSlug The repository slug
    @returns {Array} An array of the fellow objects that maintain to the repository
    */

  }, {
    key: "maintainsRepository",
    value: function maintainsRepository(repoSlug) {
      return this.list.filter(function (fellow) {
        return fellow.ensureRepository(repoSlug).maintains;
      });
    }
    /**
    Get all fellows who author a particular repository
    @static
    @param {String} repoSlug The repository slug
    @returns {Array} An array of the fellow objects that author to the repository
    */

  }, {
    key: "authorsRepository",
    value: function authorsRepository(repoSlug) {
      return this.list.filter(function (fellow) {
        return fellow.ensureRepository(repoSlug).authors;
      });
    }
  }, {
    key: "list",
    get: function get() {
      if (this._list == null) this._list = [];
      return this._list;
    }
  }]);

  function Fellow(value) {
    _classCallCheck(this, Fellow);

    if (!this.emails) this.emails = [];
    this.set(value);
  }
  /**
  Update our fellow with the passed value
  @param {String|Object} fellow - A string in the format of "Benjamin Lupton <b@lupton.cc> (https://balupton.com)" or an object with properties
  @returns {this}
  */


  _createClass(Fellow, [{
    key: "set",
    value: function set(fellow) {
      var _this2 = this;

      // String format
      // Benjamin Lupton <b@lupton.cc> (https://balupton.com)
      if (typeof fellow === 'string') {
        var match = /^([^<(]+)\s*(?:<(.+?)>)?\s*(?:\((.+?)\))?$/.exec(fellow);

        if (!match) {
          throw new Error('Invalid fellow string');
        }

        var name = (match[1] || '').trim() || null;
        var email = (match[2] || '').trim() || null;
        var url = (match[3] || '').trim() || null;
        if (name) this.name = name;
        if (email) this.email = email;
        if (url) this.url = url;
      } // Object Format
      else if (_typeof(fellow) === 'object') {
          var urlFields = ['url', 'homepage', 'web', 'githubUrl', 'twitterUrl', 'facebookUrl'];
          Object.keys(fellow).forEach(function (key) {
            if (key[0] === '_') return; // skip if private

            var value = fellow[key] || null;

            if (value) {
              // if not a url field, e.g. name or email
              if (urlFields.indexOf(key) === -1) {
                _this2[key] = value;
              } // if any of the url fields, redirect to url setter
              else {
                  _this2.url = value;
                }
            }
          });
        } else {
          throw new Error('Invalid fellow input');
        }

      return this;
    } // -----------------------------------
    // Repos

    /**
    Ensures that the github repository exists on the class
    @private
    @param {String} slug - Github repository slug (e.g. "bevry/projectz")
    @returns {Object} The repository object
    */

  }, {
    key: "ensureRepository",
    value: function ensureRepository(slug) {
      if (this._Repositories == null) this._Repositories = {};
      if (this._Repositories[slug] == null) this._Repositories[slug] = {
        contributes: false,
        maintains: false,
        authors: false
      };
      return this._Repositories[slug];
    }
    /**
    Get all added github repository slugs that this fellow contributes to
    @type {Array}
    */

  }, {
    key: "contributesRepository",

    /**
    Make note that this fellow contributes to this repository slug
    @param {String} slug - The github repository slug that this user contributes to
    @returns {this}
    */
    value: function contributesRepository(slug) {
      this.ensureRepository(slug).contributes = true;
      return this;
    }
    /**
    Get all added github repository slugs that this fellow maintains
    @type {Array}
    */

  }, {
    key: "maintainsRepository",

    /**
    Make note that this fellow maintains this repository slug
    @param {String} slug - The github repository slug that this user maintains
    @returns {this}
    */
    value: function maintainsRepository(slug) {
      this.ensureRepository(slug).maintains = true;
      return this;
    }
    /**
    Get all added github repository slugs that this fellow authors
    @type {Array}
    */

  }, {
    key: "authorsRepository",

    /**
    Make note that this fellow authors this repository slug
    @param {String} slug - The github repository slug that this user authors
    @returns {this}
    */
    value: function authorsRepository(slug) {
      this.ensureRepository(slug).authors = true;
      return this;
    }
  }, {
    key: "contributedRepositories",
    get: function get() {
      var _this3 = this;

      return Object.keys(this._Repositories || {}).filter(function (slug) {
        var repo = _this3._Repositories[slug];
        return repo && repo.contributes;
      });
    }
  }, {
    key: "maintainedRepositories",
    get: function get() {
      var _this4 = this;

      return Object.keys(this._Repositories || {}).filter(function (slug) {
        var repo = _this4._Repositories[slug];
        return repo && repo.maintains;
      });
    }
  }, {
    key: "authoredRepositories",
    get: function get() {
      var _this5 = this;

      return Object.keys(this._Repositories || {}).filter(function (slug) {
        var repo = _this5._Repositories[slug];
        return repo && repo.authors;
      });
    }
  }]);

  return Fellow;
}(); // Export


module.exports = Fellow;